<?php

use Delo\DeloSettings;
use Delo\DeloLog;
use Delo\DeloRequirements;

/**
 * Class ControllerExtensionPaymentDelo
 * Расширение для связи с платежным шлюзом, админка
 */
class ControllerExtensionPaymentDelo extends Controller
{
    /** @var Registry */
    protected $registry;
    /** @var DeloSettings */
    private $settings;
    /** @var DeloLog */
    private $log;
    /** @var DeloRequirements */
    private $requirements;
    /** @var string */
    private $tokenField;
    /** @var string */
    private $linkExtension;

    /**
     * ControllerExtensionPaymentDelo constructor.
     * @param Registry $registry
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        // Подключение нужных классов
        $this->load->language('extension/payment/delo');
        $this->load->model('localisation/order_status');
        $this->load->library('Delo/DeloLog');
        $this->load->library('Delo/DeloErrors');
        $this->load->library('Delo/DeloRequirements');
        $this->load->library('Delo/DeloSettings');
        $this->log = $this->registry->get('DeloLog');
        $this->requirements = $this->registry->get('DeloRequirements');
        $this->settings = $this->registry->get('DeloSettings');
        $this->tokenField = $this->determineWhereToken();
        $this->linkExtension = $this->linkExtension();
    }

    /**
     * Настройки расширения
     * @throws Exception
     */
    public function index()
    {
        $this->document->setTitle($this->language->get('heading_title') . ' - ' . $this->language->get('subheading_settings'));
        $data = [
            'breadcrumbs'                            => $this->breadcrumbsIndex(),
            'header'                                 => $this->load->controller('common/header'),
            'column_left'                            => $this->load->controller('common/column_left'),
            'footer'                                 => $this->load->controller('common/footer'),
            'cancel'                                 => $this->url->link($this->linkExtension, $this->tokenField . '=' . $this->session->data[$this->tokenField] . '&type=payment', true),
            'action'                                 => $this->url->link('extension/payment/delo/save', $this->tokenField . '=' . $this->session->data[$this->tokenField], true),
            'back_url_s_label_hint'                  => $this->settings->setRealDomain($this->language->get('back_url_s_label_hint'), $this->language->get('example_domain')),
            'back_url_s_placeholder'                 => $this->settings->setRealDomain($this->language->get('back_url_s_placeholder'), $this->language->get('example_domain')),
            'back_url_f_label_hint'                  => $this->settings->setRealDomain($this->language->get('back_url_f_label_hint'), $this->language->get('example_domain')),
            'back_url_f_placeholder'                 => $this->settings->setRealDomain($this->language->get('back_url_f_placeholder'), $this->language->get('example_domain')),
            'lang_options'                           => $this->settings->optionsLang(),
            'currency_options'                       => $this->settings->optionsCurrency(),
            'prefix'                                 => $this->settings->prefixSettings,
            'error_warning'                          => '',
            'order_statuses'                         => $this->model_localisation_order_status->getOrderStatuses(),
        ];
        $data = array_merge($data, $this->settings->getSettingsValue());
        $data = array_merge($data, $this->settings->getErrorsFromSession());
        $data = array_merge($data, $this->language->all());
        $this->response->setOutput($this->load->view('extension/payment/delo', $data));
    }

    /**
     * Вызывается в момент установки расширения
     * @throws Exception
     */
    public function install()
    {
        $this->log->log('Устанавливаем ' . $this->language->get('heading_title') . '... ');
        $this->requirements->setLog($this->log);
        if (!$this->requirements->validateRequirementsBeforeInstall()) {
            /**
             * Т.к. до нашего метода Opencart выполняет свои процедуру установки, то для случая
             * если наше расширение не может быть установлено, надо откатить эти процедуры
             */
            $this->load->controller('extension/extension/payment/uninstall');
        } else {
            $this->requirements->createTableTransactionPayment();
            $this->requirements->optimizationCommit();
            $this->settings->setDefaultValueSettings();
        }
    }

    /**
     * Вызывается в момент удаления расширения
     */
    public function uninstall()
    {
        $this->requirements->dropTableTransactionPayment();
        $this->requirements->optimizationRollback();
        $this->log->log('Удалили ' . $this->language->get('heading_title'));
    }

    /**
     * Action куда отправляются данные с формы настроек
     * @throws Exception
     */
    public function save()
    {
        if ($this->request->server['REQUEST_METHOD'] == 'POST') {
            $data = $this->settings->filter($this->request->post);
            if ($this->settings->validate($data)) {
                $this->settings->save($data);
                $this->response->redirect($this->url->link($this->linkExtension, $this->tokenField . '=' . $this->session->data[$this->tokenField] . '&type=payment', true));
            } else {
                $this->settings->setValueInSession($this->request->post);
                $this->settings->setErrorsInSession();
                $errors = $this->settings->errors->getErrors();
                $errors = array_keys($errors);
                $firstError = array_shift($errors);
                $field = $this->settings->prefixSettings . str_replace('_error', '', $firstError);
            }
        }
        $this->response->redirect($this->url->link('extension/payment/delo', $this->tokenField . '=' . $this->session->data[$this->tokenField], true) . (!empty($field) ? '#' . $field : ''));
    }

    /**
     * Хлебные крошки для страницы с настройками
     * @return array
     */
    private function breadcrumbsIndex()
    {
        return [
            [
                'text' => $this->language->get('text_home'),
                'href' => $this->url->link('common/dashboard', $this->tokenField . '=' . $this->session->data[$this->tokenField], true),
            ],
            [
                'text' => $this->language->get('text_extension'),
                'href' => $this->url->link($this->linkExtension, $this->tokenField . '=' . $this->session->data[$this->tokenField] . '&type=payment', true)
            ],
            [
                'text' => $this->language->get('subheading_settings'),
                'href' => $this->url->link('extension/payment/delo', $this->tokenField . '=' . $this->session->data[$this->tokenField], true)
            ],
        ];
    }

    /**
     * Определяет и возвращает название параметра в котором передается авторизационный токен
     * @return string
     */
    private function determineWhereToken()
    {
        $version = $this->requirements->getMajorVersionCMS();
        if ($version == 2) {
            return 'token';
        } elseif ($version == 3) {
            return 'user_token';
        }
        // @todo: если первая версия CMS, что нас ждет там
        return 'token';
    }

    /**
     * Возвращает ссылку на раздел расширении в зависимости от версии CMS
     * @return string
     */
    private function linkExtension()
    {
        $version = $this->requirements->getMajorVersionCMS();
        if ($version == 2) {
            return 'extension/extension';
        } elseif ($version == 3) {
            return 'marketplace/extension';
        }
        // @todo: если первая версия CMS, что нас ждет там
        return 'extension/extension';
    }
}
