<?php
namespace Delo;

use \Registry;
use \Language;
use \DB;

/**
 * Проверки требовании перед установкой расширения
 */
class DeloRequirements
{
    const PHP_VERSION = '5.4';

    /** @var DeloErrors */
    private $errors;
    /** @var DeloLog|null */
    private $log = null;
    /** @var Language */
    private $language;
    /** @var DB */
    private $db;
    /** @var int */
    private $majorVersionCMS = 0;

    /**
     * @param Registry $registry
     */
    public function __construct($registry)
    {
        $loader = $registry->get('load');
        $loader->library('Delo/DeloErrors');
        $this->errors = $registry->get('DeloErrors');
        $this->language = $registry->get('language');
        $this->db = $registry->get('db');
    }

    /**
     * Проверки перед установкой расширения, если все ок, возвращает true, иначе false
     * @return bool
     */
    public function validateRequirementsBeforeInstall()
    {
        // Проверка версии PHP
        if (!version_compare(PHP_VERSION, self::PHP_VERSION, '>=')) {
            $this->errors->setError('На вашем сервере до сих пор использую PHP ' . PHP_VERSION . ', для расширения "'
                . $this->language->get('heading_title') . '" необходима минимум версия: ' .  self::PHP_VERSION, 0);
            return false;
        }
        // Проверка установлен ли openSSL
        if (!extension_loaded('openssl')) {
            $this->errors->setError('Для расширения "' . $this->language->get('heading_title') . '" необходима библиотека openSSL', 1);
            return false;
        }
        // Пишем ошибки в лог
        $this->writeErrorInLog();
        return true;
    }

    /**
     * Небольшая оптимизация, делаем индекс на таблице настроек
     */
    public function optimizationCommit()
    {
        $this->db->query('CREATE INDEX k_store_code ON ' . DB_PREFIX . 'setting (store_id, code)');
    }

    /**
     * Откатываем оптимизацию
     */
    public function optimizationRollback()
    {
        $this->db->query('DROP INDEX k_store_code ON ' . DB_PREFIX . 'setting');
    }

    /**
     * Создаем таблицу под логи
     */
    public function createTableTransactionPayment()
    {
        $q = 'CREATE TABLE IF NOT EXISTS ' . DB_PREFIX . 'delo_transaction_payment (
                trx_id CHAR(32) NOT NULL,
                order_id INTEGER NOT NULL,
                info TEXT NOT NULL,
                ts TIMESTAMP NOT NULL,
                PRIMARY KEY (order_id, trx_id)
            )';
        $this->db->query($q);
    }

    /**
     * Удаляем таблицу под логи
     */
    public function dropTableTransactionPayment()
    {
        $this->db->query('DROP TABLE IF EXISTS ' . DB_PREFIX . 'delo_transaction_payment');
    }

    /**
     * Возвращает мажорную часть версии Opencart-а
     * Например для версии 3.0.3.2 это будет 3 и т.д.
     * @return int мажорная часть версии
     */
    public function getMajorVersionCMS()
    {
        if (empty($this->majorVersionCMS)) {
            $versionArr = explode('.', VERSION);
            $this->majorVersionCMS = (int)$versionArr[0];
        }
        return $this->majorVersionCMS;
    }

    /**
     * Пишет ошибки в лог
     */
    public function writeErrorInLog()
    {
        if (empty($this->log)) {
            return ;
        }
        $errors = $this->errors->getErrors();
        if (!empty($errors)) {
            $message = 'Errors: ' . PHP_EOL;
            foreach ($errors as $error) {
                $message .= $error . PHP_EOL;
            }
            $this->log->log($message);
        }
    }

    /**
     * @param DeloLog|null $log объект лога
     */
    public function setLog($log)
    {
        $this->log = $log;
    }
}
